"""Preview and restore AWS SSM params backed-up by the event-driven Lambda function.

Restores SSM Parameters to their state at a given time. Preview is
just a dry run without actual restore. Latest is always relative to
the point in time (checktime). Works for just one key or a path with a
bunch. You can choose whether to operate on the path recursively
(default False).

Typical usage example (note trailing slash for path/):

from ssmbak.restore.actions import Path

from datetime import datetime, timezone

point_in_time = datetime(2023, 8, 3, 21, 9, 31, tzinfo=timezone.utc)

path = Path("/some/ssm/path/", point_in_time, "us-west-2", mys3bucket, recurse=True)

previews = path.preview()

path.restore()  #  == previews
"""

import logging
from datetime import datetime, timezone

from ssmbak.restore.aws import Resource
from ssmbak.typing import Preview, Version

logger = logging.getLogger(__name__)


class Path(Resource):
    """An s3/ssm key or a path to restore to a point in time.

    SSM Parms will be restored to their values at checktime. If params
    were deleted at that time, they will be deleted upon Path.restore().
    The lambda will back up any ssm change to exactly the same key in
    the configured s3 bucket.  Multiple keys not in the same path will
    have to instantiate a Path object for each one.

    Attributes:
      :param name: A string of the ssm/s3 key or path
      :param checktime: the point in time for which to retrieve relative latest version
      :param recurse: A boolean to operate on all paths/keys under name/
      :param versions: A cache used for preview/restore, starts empty
    """

    def __init__(  # pylint: disable=too-many-arguments
        self,
        name: str,
        checktime: datetime,
        region: str,
        bucketname: str,
        recurse=False,
    ):
        """Initializes path/key with the region, backup bucket, and point in time.

        The bucket and params need to be accessible using the same
        region. Internally, all datetimes are tz aware, using UTC.

        Args:
          name: ssm/s3 key or path
          checktime: the point in time for which to retrieve relative latest version
          region: The AWS region for params and bucket access
          bucketname: The same bucket that the lambda writes to.
          recurse: operate on all paths/keys under name/
        """
        self.name = name  # .rstrip("/")
        self.checktime = checktime
        self.recurse = recurse
        self.versions = {}
        super().__init__(region, bucketname)

    def __repr__(self):
        return (
            f"{self.__class__.__name__}({self.name}, "
            f"{self.checktime.strftime('%Y-%m-%dT%H:%M:%SZ')}, "
            f"{self.region}, {self.bucketname})"
        )

    def get_names(self) -> list[str]:
        """Get the names of the latest versions.

        Seeds the version cache self.versions along the way.

        Returns:
          A list of version names only, e.g.

          ["/some/key", "/some/other/key"]
        """
        versions = self.get_versions()
        names = list(set(versions))
        if self.name in names and not self.recurse:  # if it's a key and not a path
            return [self.name]
        return names

    def get_versions(self) -> dict[str, Version]:
        """Grabs the verbose versions most recent relative to checktime.

        Keyed by s3/ssm key name.

        Returns:
        Returns the self.versions cache if it's not empty, populates
        it otherwise. Contains more infomration that ends up in
        preview.

          {
              "/testyssmbak/XHG0Y1": {
                  "Body": "4PPS8T",
                  "ETag": '"3149f5a99287b0e05fe34446b4fbe054"',
                  "IsLatest": False,
                  "Key": "/testyssmbak/XHG0Y1",
                  "LastModified": datetime.datetime(
                      2024, 6, 8, 21, 45, 22, tzinfo=tzutc()
                  ),
                  "Owner": {
                      "DisplayName": "webfile",
                      "ID": "029ejf2ienc09",
                  },
                  "Size": 6,
                  "StorageClass": "STANDARD",
                  "VersionId": "OMY7u3ey3H6ACQEbne96zQ",
                  "tagset": {
                      "ssmbakDescription": "fancy " "description",
                      "ssmbakTime": "1659560971",
                      "ssmbakType": "SecureString",
                  },
              }
          }

        """
        if self.versions:
            versions = self.versions
        else:
            versions = self._get_versions(
                self.name,
                self.checktime,
                recurse=self.recurse,
            )
            self.versions = versions
        return versions

    def preview(self) -> list[Preview]:
        """Shows what would be restored.

        Returns:
          A list of dicts, one for each ssm/s3 key, with concise
          information about the latest versions to be restored
          relative to checktime.

          [
              {
                  "Description": "fancy description",
                  "Modified": datetime.datetime(
                      2022, 8, 3, 21, 9, 31, tzinfo=datetime.timezone.utc
                  ),
                  "Name": "/testyssmbak/08D2SR",
                  "Type": "SecureString",
                  "Value": "C2FMGS",
              }
          ]
        """

        names = self.get_names()
        previews = [self.preview_key(name) for name in names]
        return sorted(previews, key=lambda d: d["Name"])

    def restore(self) -> list[Preview]:
        """Restore parameters to their state at time,

        It uses self.preview's returned values to actually perform the
        restore. Deleted params are handled efficiently in batches.

        Returns:
          A list of dicts, one for each ssm/s3 key, with concise
          information about the latest versions to be restored
          relative to checktime.

          [
              {
                  "Description": "fancy description",
                  "Modified": datetime.datetime(
                      2022, 8, 3, 21, 9, 31, tzinfo=datetime.timezone.utc
                  ),
                  "Name": "/testyssmbak/08D2SR",
                  "Type": "SecureString",
                  "Value": "C2FMGS",
              },
              {
                  "Description": "fancy description",
                  "Modified": datetime.datetime(
                      2022, 8, 3, 21, 9, 31, tzinfo=datetime.timezone.utc
                  ),
                  "Name": "/testyssmbak/19F3TS",
                  "Type": "SecureString",
                  "Value": "D3GNFT",
              },
          ]
        """
        params = self.preview()
        self._ssm_del_multi(
            [x["Name"] for x in params if "Deleted" in x and x["Deleted"] is True]
        )
        for param in [x for x in params if "Deleted" not in x]:
            self._restore_preview(param)
        return params

    def preview_key(self, name: str) -> Preview:
        """Shows what would be restored for the single s3/ssm key.

        Args:
          name: the s3/ssm key

        Returns:
          A dict with concise information about the key.
          {
              "Description": "fancy description",
              "Deleted": True,
              "Modified": datetime.datetime(
                  2022, 8, 3, 21, 9, 31, tzinfo=datetime.timezone.utc
              ),
              "Name": "/testyssmbak/5M9UOV",
              "Type": "SecureString",
              "Value": "318Z27",
          }
        """
        res = {"Name": name}
        version = self.get_latest_version(name)
        if "Deleted" in version:
            res["Deleted"] = True
            res["Modified"] = version["LastModified"]
        elif version:
            res["Value"] = version["Body"]
            tagset = version["tagset"]
            res["Type"] = tagset["ssmbakType"]
            res["Modified"] = datetime.fromtimestamp(
                int(tagset["ssmbakTime"]), tz=timezone.utc
            )
            if "ssmbakDescription" in tagset:
                res["Description"] = tagset["ssmbakDescription"]
        else:
            logger.warning(
                "Key %s doesn't have a version before %s", name, self.checktime
            )
        return res

    def get_latest_version(self, name: str) -> Version:
        """Gets the concise latest version of a particular s3/ssm key.

        Returns from the self.versions cache if it includes the key,
        populates it otherwise.

        Args:
          name: the s3/ssm key

        Returns:
          A dict with concise information about the key.
          {
              "Description": "fancy description",
              "Deleted": True,
              "Modified": datetime.datetime(
                  2022, 8, 3, 21, 9, 31, tzinfo=datetime.timezone.utc
              ),
              "Name": "/testyssmbak/5M9UOV",
              "Type": "SecureString",
              "Value": "318Z27",
          }
        """
        if name in self.versions:
            version = self.versions[name]
        else:
            all_versions = self._get_versions(name, self.checktime)
            try:
                version = all_versions[name]
                self.versions[name] = version
            except KeyError:
                version = {}
        try:
            if "Deleted" not in version and "Body" not in version:
                version["Body"] = self._get_version_body(name, version["VersionId"])
        except (IndexError, KeyError):
            logger.debug("No versions")
            version = {}
        return version



"""Handles interactions with AWS APIs.

Not to be handled directly.
"""

import logging
import os
from datetime import datetime, timezone
from functools import cached_property
from typing import ClassVar, Union

import boto3
import botocore
from botocore.exceptions import ClientError

from ssmbak.typing import Version

logger = logging.getLogger(__name__)


class Resource:
    """Parent to actions.Path.

    Interface between what's in SSM now and corresponding s3 backups
    from the Lambda function. The region for SSM params and bucket
    access need to be the same.

    Attributes:
      region: The AWS region for params and bucket access.
      bucketname: The same bucket that the lambda writes to.
      _CALLS: class attribute strictly for testing efficiency of AWS calls
    """

    _CALLS: ClassVar[dict[str, int]] = {"tags": 0, "versions": 0, "version_objects": 0}

    def __init__(self, region, bucketname):
        self.region = region
        self.bucketname = bucketname

    @classmethod
    def clear_call_cache(cls) -> None:
        """Reset call counts between tests."""
        cls._CALLS = {"tags": 0, "versions": 0, "version_objects": 0}

    @classmethod
    def get_calls(cls) -> dict[str, int]:
        """Access call counts from tests."""
        return cls._CALLS

    @cached_property
    def s3(self) -> boto3.client:
        """boto3 s3 client. There should only be one."""
        return boto3.client(
            "s3", endpoint_url=os.getenv("AWS_ENDPOINT"), region_name=self.region
        )

    @cached_property
    def s3res(self) -> boto3.client:
        """boto3 s3 resource for backup contents. There should only be one."""
        return boto3.resource(
            "s3", endpoint_url=os.getenv("AWS_ENDPOINT"), region_name=self.region
        )

    @cached_property
    def ssm(self) -> boto3.client:
        """boto3 ssm client. There should only be one."""
        return boto3.client(
            "ssm", endpoint_url=os.getenv("AWS_ENDPOINT"), region_name=self.region
        )

    def _key_versions(self, versions: list[dict]) -> dict[str, Version]:
        """Turns a list of dicts into a dict of dicts keyed by s3/param key.

        Arguments:
          versions: un-keyed list of versions as returned by AWS
        """
        keyed_versions = {}
        for name in {x["Key"] for x in versions}:
            key_versions = [x for x in versions if x["Key"] == name]
            keyed_versions[name] = key_versions[0]
        return keyed_versions

    def _tagtime(self, version: dict) -> datetime:
        """Extracts datetime from the backup version.

        Corresponds to the time of the event and not necessary when it
        was backed-up, e.g. if there was some failures that held event
        processing up in a queue.

        In the case of deleted versions, no tags can be written so we
        just use the time of backup (LastModified).

        Arguments:
          version: dict of s3 version with processed tagset.
        """
        try:
            ssmbak_time = datetime.fromtimestamp(int(version["tagset"]["ssmbakTime"]))
            tagtime = ssmbak_time.astimezone(timezone.utc)
        except KeyError:
            tagtime = version["LastModified"]
        return tagtime

    def _get_tagset(self, name: str, versionid: str) -> dict[str, str]:
        """Get the tagset from S3 for the object version, using time
        of original event not backup.

        Arguments:
          name: name of the s3 object/ssm param in question
          versionid: s3 object versionid

        Returns:
        {
            "ssmbakTime": "1659560971",
            "ssmbakType": "SecureString",
            "ssmbakDescription": "fancy description", --OPTIONAL
        }
        """
        try:
            logger.debug("actually getting tagset for %s %s", name, versionid)
            Resource._CALLS["tags"] += 1
            tagset = self.s3.get_object_tagging(
                Bucket=self.bucketname, Key=name, VersionId=versionid
            )["TagSet"]
            nice_tagset = {x["Key"]: x["Value"] for x in tagset}
        except ClientError as e:
            if e.response["Error"]["Code"] in ["MethodNotAllowed"]:
                nice_tagset = {}
            else:
                raise e
        return nice_tagset

    def _make_ssm_kwargs(self, param: dict) -> dict[str, Union[str, bool]]:
        """Preps the kwargs for boto3 client ssm.put_parameter().

        Arguments:
          param: dict as generated by preview

        Returns:
        {
            "Name": "/testyssmbak/AP66LQ",
            "Value": "RUL38Y",
            "Type": "SecureString",
            "Overwrite": True,
            "Description": "fancy description",
        }
        """
        kwargs = {
            "Name": param["Name"],
            "Value": param["Value"],
            "Type": param["Type"],
            "Overwrite": True,
        }
        if "Description" in param:
            kwargs["Description"] = param["Description"]
        return kwargs

    def _restore_preview(self, param: dict) -> None:
        """Sets the ssm param to the desired state.

        Arguments:
          param: dict as generated by preview
        """
        if "Deleted" in param and param["Deleted"]:
            self.ssm.delete_parameter(Name=param["Name"])
        else:
            ssm_kwargs = self._make_ssm_kwargs(param)
            self.ssm.put_parameter(**ssm_kwargs)

    def _ssm_del_multi(self, names: list) -> None:
        """Delete SSM Params efficiently"""
        batch_size = 10
        chunks = [names[x : x + batch_size] for x in range(0, len(names), batch_size)]
        for chunk in chunks:
            logger.debug("deleting %s", chunk)
            self.ssm.delete_parameters(Names=chunk)

    def _ssmgetpath(self, path: str, recurse=False) -> dict[str, Version]:
        """Gets params currently in place.

        Needed to determine what to delete when _getting_versions.

        Arguments:
          path: can correspond to just one key
          recurse: A boolean to operate on all paths/keys under path/

        Returns:
          The same format used everywhere, keyed by param name.

          {
              "/testyssmbak/82P11M": {
                  "Name": "/testyssmbak/82P11M",
                  "Type": "String",
                  "Value": "ES9IT7",
                  "Version": 2,
                  "LastModifiedDate": datetime.datetime(
                      2024, 6, 9, 9, 37, 34, 202000, tzinfo=tzlocal()
                  ),
                  "ARN": "arn:aws:ssm:us-west-2:000000000000:parameter/testyssmbak/82P11M",
                  "DataType": "text",
              },
          }
        """
        paginator = self.ssm.get_paginator("get_parameters_by_path")
        paginated = paginator.paginate(
            Path=path, Recursive=recurse, WithDecryption=True
        )
        result = paginated.build_full_result()
        keyed_params = {}
        if result["Parameters"]:
            params = result["Parameters"]
        elif not path.endswith("/"):
            try:
                param = self.ssm.get_parameter(Name=path, WithDecryption=True)[
                    "Parameter"
                ]
                params = [param]
            except KeyError:  # nothing found
                params = []
        else:
            params = []
        for name in {x["Name"] for x in params}:
            keyed_params[name] = [x for x in params if x["Name"] == name][0]
        return keyed_params

    def _get_object_versions(self, key: str) -> botocore.paginate.PageIterator:
        """Get a versions iterator from AWS

        Arguments:
          key: a single s3 key

        They come a thousand at a time.
        """
        logger.debug("actually getting versions for %s", key)
        paginator = self.s3.get_paginator("list_object_versions")
        # they come back most recent (LastModified) first
        Resource._CALLS["versions"] += 1
        return paginator.paginate(Bucket=self.bucketname, Prefix=key)

    def _get_versions(
        self, key: str, checktime: datetime, recurse: bool = False
    ) -> dict[str, Version]:
        """Efficiently looks for the version most recently backed-up before checktime.

        The objects come from AWS a thousand at a time, but only with
        modified times corresponding to when they were backed-up (LastModified) and
        not when the original event was reported. Typically it's less
        than a minute, but in the case of an outage it might be
        longer. To be safe, we check the event times, encoded in s3
        tags by the Lambda (ssmbakTime).

        Arguments:
          key: a single s3 key or path
          checktime: the point in time for which to retrieve relative latest version
          recurse: operate on all paths/keys under key/

        Returns:
          The same keyed versions as everywhere.

          {
              "/testyssmbak/88JCRX": {
                  "ETag": '"9d2f3ea8da7b4feba87aeb4da1fcb5e0"',
                  "Size": 6,
                  "StorageClass": "STANDARD",
                  "Key": "/testyssmbak/88JCRX",
                  "VersionId": "vuyAs6cfwwSbMUi4o8O1qA",
                  "IsLatest": True,
                  "LastModified": datetime.datetime(
                      2024, 6, 9, 16, 45, 4, tzinfo=tzutc()
                  ),
                  "Owner": {
                      "DisplayName": "webfile",
                      "ID": "75aaa08ebf849d0f8e7faeebf76c078efc7c6caea54ba06a",
                  },
                  "tagset": {"ssmbakTime": "1717951504", "ssmbakType": "SecureString"},
              },
          }
        """
        versions = []
        paginated = self._get_object_versions(key)
        there_nows = self._ssmgetpath(key, recurse=recurse)
        for param_page in paginated:
            to_extend = []
            try:
                for deleted_version in param_page["DeleteMarkers"]:
                    # only need to delete if it's there now
                    if deleted_version["Key"] in there_nows:
                        deleted_version["Deleted"] = True
                        to_extend.append(deleted_version)
            except KeyError:
                logger.debug("no delete markers")
            try:
                to_extend.extend(param_page["Versions"])
            except KeyError:
                logger.debug("no versions")
            if not recurse or not key.endswith("/"):
                if key in [x["Key"] for x in to_extend + versions]:
                    to_extend = [x for x in to_extend if x["Key"] == key]
                else:
                    n = key.count("/")
                    to_extend = [x for x in to_extend if x["Key"].count("/") == n]
            for version in to_extend:
                if version["Key"] not in [x["Key"] for x in versions]:
                    version["tagset"] = self._get_tagset(
                        version["Key"], version["VersionId"]
                    )
                    if self._tagtime(version) <= checktime:
                        versions.append(version)
        return self._key_versions(versions)

    def _get_version_body(self, name: str, versionid: str) -> str:
        """Uses s3 object resource to get the contents of the version.

        Should only be run after all last versions are got.

        Arguments:
          name: single s3 key
          versionid: s3 object versionid

        Returns:
          String of the backed-up ssm paramter's value.
        """
        Resource._CALLS["version_objects"] += 1
        logger.debug("actually getting contents for %s", name)
        version = self.s3res.ObjectVersion(self.bucketname, name, versionid)
        try:
            res = version.get()  # boto3 issue #832
            body = self._get_contents(res)
        except ClientError as e:
            # NoSuchVersion to accommodate localstack
            if e.response["Error"]["Code"] in ["MethodNotAllowed", "NoSuchVersion"]:
                body = ""
            else:
                raise e
        return body

    def _get_contents(self, version: dict) -> str:
        """Reads and decodes the s3 object's body."""
        stuff = ""
        try:
            stuff = version["Body"].read().decode("utf-8").strip()
        except ClientError as e:
            if e.response["Error"]["Code"] == "MethodNotAllowed":
                # it was deleted
                pass
        return stuff



"""A CLI interface to the ssmbak library. It's barely a CLI interface
and more of a wrapper to the ssmbak lib. Its main function is to
restore an ssm path, either key or path (recursive or not) to their
state at a point in time (checktime).
"""

import argparse
import logging
import os
import pprint
import sys
from datetime import datetime, timezone
from importlib.metadata import metadata, version
from textwrap import wrap

from botocore.exceptions import ClientError, NoRegionError
from prettytable import PrettyTable

from ssmbak.cli import helpers
from ssmbak.restore.actions import Path

logger = logging.getLogger(__name__)
pp = pprint.PrettyPrinter(indent=4)
parser = argparse.ArgumentParser(
    description=__doc__,
    formatter_class=argparse.RawDescriptionHelpFormatter,
    prog=os.path.basename(__file__),
)
parser.add_argument("command", help="one of preview or restore")
parser.add_argument(
    "path",
    help="ssm/s3 path/ or key",
)
parser.add_argument(
    "checktime",
    help="""point-in-time (UTC) to retrieve latest values, e.g. 2022-08-03T21:10:00""",
)
parser.add_argument(
    "-r",
    "--region",
    help="aws region, default same as boto/awscli",
    default="",
)
parser.add_argument(
    "-b",
    "--bucket",
    help="bucket that the lambda backs up to",
    default="",
)
parser.add_argument(
    "-R",
    "--recursive",
    action="store_true",
    default=False,
    help="recursive, only for actual path not key",
)
parser.add_argument(
    "-v",
    "--verbose",
    action="store_true",
    default=False,
    help="increase logging verbosity",
)
parser.add_argument(
    "--version",
    action="version",
    version=f"{metadata('ssmbak')['Name']} {version('ssmbak')}",
    help="print the version and quit",
)
args = parser.parse_args()
checktime = datetime.strptime(args.checktime, "%Y-%m-%dT%H:%M:%S").replace(
    tzinfo=timezone.utc
)


def main():
    """Checks for necessary confs, invokes ssmbak method Path.command,
    and tries to print out a nice table of results..
    """
    if args.verbose:
        logging.basicConfig(level=logging.INFO)
    try:
        region = helpers.sort_region(args.region)
        bucketname = helpers.sort_bucket(args.bucket, region)
        result = _do_path(bucketname, region)
        _print_outs(result)
    except KeyboardInterrupt:
        logger.fatal("Interrupted")
        sys.exit(1)
    except ClientError as e:
        logger.fatal(
            "%s: %s", e.response["Error"]["Code"], e.response["Error"]["Message"]
        )
        if e.response["Error"]["Code"] == "ParameterNotFound":
            logger.fatal("Couldn't find SSM bucket param set by the stack.")
        sys.exit(1)
    except NoRegionError as e:
        logger.fatal(e)
        logger.fatal(
            "Specify a region 1) as an argument, "
            "2) using env var AWS_DEFAULT_REGION, or "
            "3) region= in ~/.aws/config."
        )
        sys.exit(1)


def _do_path(bucketname, region):
    path = Path(args.path, checktime, region, bucketname, recurse=args.recursive)
    return getattr(path, args.command)()


def _limit_string(val, limit=40):
    return "\n".join(wrap(str(val), limit))


def _print_outs(outs):
    try:
        keys = set().union(*outs)
        for out in outs:
            for key in keys - out.keys():
                out[key] = ""
        table = PrettyTable()
        headings = list(outs[0])
        table.field_names = headings
        table.align = "l"
        for out in outs:
            if out:
                table.add_row([_limit_string(out[x]) for x in headings])
        if len(table.rows) > 0:
            print(table)
    except (TypeError, AttributeError):  # pprint if table trouble
        pp.pprint(outs)
    except (KeyError, IndexError):  # for none
        pass



"""Creates a cloudformation stack with resources needed for
event-driven backup of SSM Param changes.
"""

import argparse
import logging
import pprint
import sys
from importlib.metadata import version
from pathlib import Path

from botocore.exceptions import ClientError, NoRegionError, ParamValidationError

from ssmbak.cli import helpers
from ssmbak.cli.cfn import Stack

logger = logging.getLogger(__name__)
pp = pprint.PrettyPrinter(indent=4)
parser = argparse.ArgumentParser(
    description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
)
parser.add_argument(
    "stackname",
    help="The name of the Cloudformation stack to be created",
)
parser.add_argument(
    "command",
    help="create | bucketname | lambdaname | region | status",
)
parser.add_argument(
    "-r",
    "--region",
    help="aws region, defaults to boto's default",
    default="",
)
# pylint: disable=duplicate-code
parser.add_argument(
    "-v",
    "--verbose",
    action="store_true",
    default=False,
    help="increase logging verbosity",
)
# pylint: enable=duplicate-code
args = parser.parse_args()


def main():
    """Top-level, just to execute stack commands and handle the exceptions."""
    if args.verbose:
        logging.basicConfig(level=logging.INFO)
    try:
        region = helpers.sort_region(args.region)
        _do_cfn(region)
    # pylint: disable=duplicate-code
    except KeyboardInterrupt:
        print("Interrupted")
        sys.exit(1)
    except ClientError as e:
        logger.fatal(
            "%s: %s", e.response["Error"]["Code"], e.response["Error"]["Message"]
        )
        sys.exit(1)
    except ParamValidationError:
        logger.fatal("ParamValidationError")
        sys.exit(1)
    except NoRegionError as e:
        logger.fatal(e)
        logger.fatal(
            "Specify a region 1) as an argument, "
            "2) using env var AWS_DEFAULT_REGION, or "
            "3) region= in ~/.aws/config."
        )
        sys.exit(1)
    # pylint: enable=duplicate-code


def _do_cfn(region):
    stack = Stack(args.stackname, region)
    if args.command in ["create", "update"]:
        template_dir = Path(__file__).parent.parent
        template_file = f"{template_dir}/data/cfn.yml"
        getattr(stack, args.command)(template_file, {"Version": version("ssmbak")})
        yay = stack.watch()
        if yay and args.command == "create":
            print()
            print(f"Lambda: {stack.lambdaname}")
            print(f"Bucket: {stack.bucketname}")
            print(f"Cloudwatch log group: /aws/lambda/{stack.lambdaname}")
            print()
            print(
                "ssmbak-all will back up all ssm params if provided --do-it, "
                "list if not"
            )
            print()
        elif args.command == "create":
            print(
                "Check the error messages above. Are you sure you have enough privs"
                "to create all those resources?"
            )
    elif args.command == "bucketname":
        print(stack.bucketname)
    elif args.command == "lambdaname":
        print(stack.lambdaname)
    elif args.command == "region":
        print(stack.region)
    elif args.command == "status":
        print(stack.status())
    else:
        print("Not a command.")
